"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dispatch = dispatch;
exports.dispatchDelete = dispatchDelete;
exports.dispatchFilter = dispatchFilter;
exports.dispatchMap = dispatchMap;
exports.loadState = loadState;
exports.reducer = reducer;
exports.reduxSetup = reduxSetup;
exports.saveState = saveState;
exports.setStore = setStore;
exports.watch = watch;
exports.handleAsyncAction = exports.getStore = exports.getPath = exports.getState = exports.dispatchTransform = exports.dispatchSet = exports.dispatchPush = exports.addReducer = exports.STATE_KEY = exports.INIT = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _function = require("lodash/function");

var _pathNext = require("path-next");

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _reactRedux = require("react-redux");

var _redux = require("redux");

var _cjs = require("flatted/cjs");

var _reducers;

var actionListener,
    initialState = {},
    replacerFn,
    reviverFn,
    sessionStorageOptOut,
    silent,
    store;
var ASYNC = '@@async';
var DELETE = '@@delete';
var FILTER = '@@filter';
var INIT = '@@redux/INIT';
exports.INIT = INIT;
var MAP = '@@map';
var PUSH = '@@push';
var SET = '@@set';
var STATE_KEY = 'reduxState';
exports.STATE_KEY = STATE_KEY;
var TRANSFORM = '@@transform';
var VERSION_KEY = '@reduxEasyVersion';
var reducers = (_reducers = {}, (0, _defineProperty2["default"])(_reducers, ASYNC, function (state, payload) {
  return payload;
}), (0, _defineProperty2["default"])(_reducers, DELETE, function (state, payload) {
  return (0, _pathNext.deletePath)(state, payload);
}), (0, _defineProperty2["default"])(_reducers, FILTER, function (state, _ref) {
  var path = _ref.path,
      value = _ref.value;
  return (0, _pathNext.filterPath)(state, path, value);
}), (0, _defineProperty2["default"])(_reducers, INIT, function () {
  return null;
}), (0, _defineProperty2["default"])(_reducers, MAP, function (state, _ref2) {
  var path = _ref2.path,
      value = _ref2.value;
  return (0, _pathNext.mapPath)(state, path, value);
}), (0, _defineProperty2["default"])(_reducers, PUSH, function (state, _ref3) {
  var path = _ref3.path,
      value = _ref3.value;
  return _pathNext.pushPath.apply(void 0, [state, path].concat((0, _toConsumableArray2["default"])(value)));
}), (0, _defineProperty2["default"])(_reducers, SET, function (state, _ref4) {
  var path = _ref4.path,
      value = _ref4.value;
  return (0, _pathNext.setPath)(state, path, value);
}), (0, _defineProperty2["default"])(_reducers, TRANSFORM, function (state, _ref5) {
  var path = _ref5.path,
      value = _ref5.value;
  return (0, _pathNext.transformPath)(state, path, value);
}), _reducers);
var version;

var addReducer = function addReducer(type, fn) {
  return reducers[type] = fn;
};

exports.addReducer = addReducer;

function dispatch(type, payload) {
  if (actionListener) actionListener(type, payload);
  store.dispatch({
    type: type,
    payload: payload
  });
}
/**
 * This deletes the property at path.
 */


function dispatchDelete(path) {
  dispatch(DELETE + ' ' + path, path);
}
/**
 * This removes elements from the array at path.
 * filterFn must be a function that takes an array element
 * and returns a boolean indicating
 * whether the element should be retained.
 */


function dispatchFilter(path, filterFn) {
  if (typeof filterFn !== 'function') {
    throw new Error('dispatchFilter must be passed a function');
  }

  dispatch(FILTER + ' ' + path, {
    path: path,
    value: filterFn
  });
}
/**
 * This updates elements in the array at path.
 * mapFn must be a function that takes an array element
 * and returns new value for the element.
 */


function dispatchMap(path, mapFn) {
  if (typeof mapFn !== 'function') {
    throw new Error('dispatchMap must be passed a function');
  }

  dispatch(MAP + ' ' + path, {
    path: path,
    value: mapFn
  });
}
/**
 * This adds elements to the end of the array at path.
 */


var dispatchPush = function dispatchPush(path) {
  for (var _len = arguments.length, elements = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    elements[_key - 1] = arguments[_key];
  }

  return dispatch(PUSH + ' ' + path, {
    path: path,
    value: elements
  });
};
/**
 * This sets the value found at path to a given value.
 */


exports.dispatchPush = dispatchPush;

var dispatchSet = function dispatchSet(path, value) {
  return dispatch(SET + ' ' + path, {
    path: path,
    value: value
  });
};

exports.dispatchSet = dispatchSet;

var dispatchTransform = function dispatchTransform(path, value) {
  if (typeof value !== 'function') {
    throw new Error('dispatchTransform must be passed a function');
  }

  dispatch(TRANSFORM + ' ' + path, {
    path: path,
    value: value
  });
};

exports.dispatchTransform = dispatchTransform;

var getState = function getState() {
  if (!store) throw new Error('store is not set');
  return reviverFn(store.getState());
};

exports.getState = getState;

var getPath = function getPath(path) {
  return (0, _pathNext.getPath)(getState(), path);
}; // This is useful in tests.


exports.getPath = getPath;

var getStore = function getStore() {
  return store;
}; // This is exported to support tests.
// Some tests need this promise to be returned.


exports.getStore = getStore;

var handleAsyncAction = function handleAsyncAction(promise) {
  return promise.then(function (newState) {
    return dispatch(ASYNC, newState);
  })["catch"](function (error) {
    return console.trace(error);
  });
};

exports.handleAsyncAction = handleAsyncAction;

var identityFn = function identityFn(state) {
  return state;
};
/**
 * This is called on app startup and
 * again each time the browser window is refreshed.
 * This function is only exported so it can be accessed from a test.
 */


function loadState() {
  var cleanState = replacerFn(initialState);
  if (sessionStorageOptOut) return cleanState;
  var _window = window,
      sessionStorage = _window.sessionStorage; // not available in tests
  // If the version passed to reduxEasy does not match the version
  // last saved in sessionStorage, assume that the shape of the state
  // may have changed and revert to initialState.

  var ssVersion = sessionStorage.getItem(VERSION_KEY);

  if (String(version) !== ssVersion) {
    sessionStorage.setItem(STATE_KEY, cleanState);
    sessionStorage.setItem(VERSION_KEY, version);
    return cleanState;
  }

  var json;

  try {
    json = sessionStorage ? sessionStorage.getItem(STATE_KEY) : null;
    if (!json || json === '""') return cleanState;
    var state = (0, _cjs.parse)(json);
    var revived = reviverFn(state);
    return revived;
  } catch (e) {
    // istanbul ignore next
    if (!silent) console.error('redux-util loadState:', e.message);
    return cleanState;
  }
} // exported to support tests


function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments.length > 1 ? arguments[1] : undefined;
  var type = action.type;

  if (!type) {
    throw new Error('action object passed to reducer must have type property');
  }

  if (type.startsWith(SET) || type.startsWith(TRANSFORM) || type.startsWith(DELETE) || type.startsWith(PUSH) || type.startsWith(FILTER) || type.startsWith(MAP)) {
    var index = type.indexOf(' ');
    type = type.substring(0, index);
  } else if (type.startsWith(INIT) || type === '@@INIT') {
    type = INIT;
  }

  var fn = reducers[type];

  if (!fn) {
    throw new Error("no reducer found for action type \"".concat(type, "\""));
  }

  var newState = fn(state, action.payload) || state;

  if (newState instanceof Promise) {
    handleAsyncAction(newState);
    return state;
  }

  (0, _pathNext.deepFreeze)(newState);
  return newState;
}
/**
 * Pass an object with these properties:
 * component: top component to render
 * actionListener: function that is passed the type and payload
 *   of each dispatched action (useful for gathering analytics)
 * target: element where component should be rendered
 *   (defaults to element with id "root")
 * initialState: required object
 * replacerFn: function that is passed the state before it is saved in
 *   sessionStorage and returns the state that should actually be saved there;
 *   can be used to avoid exposing sensitive data
 * reviverFn: function that is passed the state after it is retrieved from
 *   sessionStorage and returns the state that the app should actually use;
 *   can be used to supply sensitive data that is not in sessionStorage
 * sessionStorageOptOut: optional boolean
 *   (true to not save state to session storage)
 * silent: optional boolean
 *   (true to silence expected error messages in tests)
 * version: a version string that should be changed
 *   when the shape of the state changes
 * Returns the render function.
 */


function reduxSetup(options) {
  var component = options.component;
  actionListener = options.actionListener;
  var _options$initialState = options.initialState;
  initialState = _options$initialState === void 0 ? {} : _options$initialState;
  var _options$replacerFn = options.replacerFn;
  replacerFn = _options$replacerFn === void 0 ? identityFn : _options$replacerFn;
  var _options$reviverFn = options.reviverFn;
  reviverFn = _options$reviverFn === void 0 ? identityFn : _options$reviverFn;
  sessionStorageOptOut = options.sessionStorageOptOut;
  silent = options.silent;
  var _options$version = options.version;
  version = _options$version === void 0 ? null : _options$version;
  var target = options.target || document.getElementById('root');
  var extension = window.__REDUX_DEVTOOLS_EXTENSION__;
  var enhancer = extension && extension();
  var preloadedState = loadState(); // store is only already set when a test sets it to mock store.

  if (!store) {
    store = (0, _redux.createStore)(reducer, preloadedState, enhancer);
    setStore(store); // See the video from Dan Abramov at
    // https://egghead.io/lessons/
    // javascript-redux-persisting-the-state-to-the-local-storage.

    store.subscribe((0, _function.throttle)(function () {
      return saveState(store.getState(), 1000);
    }));
    if (options.render) store.subscribe(options.render);
  }

  var render;

  if (component && target) {
    render = function render() {
      _reactDom["default"].render(_react["default"].createElement(_reactRedux.Provider, {
        store: store
      }, component), target);
    };

    render(); // initial render
  }

  return render;
}
/**
 * This function is called by reduxSetup.
 * It is only exported so it can be accessed from a test.
 */


function saveState(state) {
  if (sessionStorageOptOut) return;

  try {
    var json = (0, _cjs.stringify)(replacerFn(state));
    sessionStorage.setItem(STATE_KEY, json);
  } catch (e) {
    // istanbul ignore next
    if (!silent) console.error('redux-util saveState:', e.message);
    throw e;
  }
}

function setStore(s) {
  store = s;
}

function watch(component, watchMap) {
  function mapState(state, ownProps) {
    state = reviverFn(state);

    if (watchMap) {
      var entries = Object.entries(watchMap);
      return entries.reduce(function (props, _ref6) {
        var _ref7 = (0, _slicedToArray2["default"])(_ref6, 2),
            name = _ref7[0],
            path = _ref7[1];

        if (!path) path = name;
        props[name] = (0, _pathNext.getPath)(state, path);
        return props;
      }, {});
    }

    var list = ownProps.list,
        path = ownProps.path; // If no watchMap is passed to the watch function
    // and the component takes a prop named "path",
    // this will pass a prop named "value" to the component
    // whose value is the value of the state at that path.

    if (path) return {
      value: (0, _pathNext.getPath)(state, path)
    }; // If no watchMap is passed to the watch function
    // and the component takes a prop named "list"
    // whose value is an array of objects that each
    // have a "path" property whose value is a state path,
    // this will pass a prop named "values" to the component
    // whose value is an array of the values at those state paths.

    if (list) {
      return {
        values: list.map(function (obj) {
          return (0, _pathNext.getPath)(state, obj.path);
        })
      };
    }

    throw new Error('watched components must have a watchMap, path, or list prop');
  }

  return (0, _reactRedux.connect)(mapState)(component);
}