"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _lodash = _interopRequireDefault(require("lodash.clonedeep"));

var _react = _interopRequireDefault(require("react"));

var _reduxMockStore = _interopRequireDefault(require("redux-mock-store"));

var _reduxEasy = require("./redux-easy");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var INITIAL_STATE = {
  foo: 1,
  bar: {
    baz: 2,
    qux: ['one', 'two', 'three']
  }
};

function mockSessionStorage() {
  var storage = {
    getItem: function getItem(key) {
      return storage[key];
    },
    removeItem: function removeItem(key) {
      delete storage[key];
    },
    setItem: function setItem(key, value) {
      storage[key] = value;
    }
  };
  global.sessionStorage = storage;
}

describe('redux-easy with mock store', function () {
  var initialState;
  var version = 0; // Mocks sessionStorage.

  beforeEach(function () {
    mockSessionStorage();
    initialState = (0, _lodash["default"])(INITIAL_STATE); // Create and register a mock store which allows
    // retrieving an array of the dispatched actions in a test.

    (0, _reduxEasy.setStore)((0, _reduxMockStore["default"])([])(initialState));
    version++;
    (0, _reduxEasy.reduxSetup)({
      initialState: initialState,
      silent: true,
      version: version
    });
  });
  test('INIT', function () {
    var action = {
      type: _reduxEasy.INIT
    };
    expect((0, _reduxEasy.reducer)(initialState, action)).toEqual(initialState);
  });
  test('dispatch', function () {
    var type = '@@set';
    var payload = {
      path: 'some.path',
      value: 'some value'
    };
    (0, _reduxEasy.dispatch)(type, payload);
    var actions = (0, _reduxEasy.getStore)().getActions();
    expect(actions.length).toBe(1);

    var _actions = (0, _slicedToArray2["default"])(actions, 1),
        action = _actions[0];

    expect(action.type).toBe(type);
    expect(action.payload).toEqual(payload);
  });
  test('dispatchSet with mock store', function () {
    // Using mock store so we can retrieve actions.
    //reduxSetup({initialState, mock: true, silent: true});
    var path = 'some.deep.path';
    var value = 'some value';
    (0, _reduxEasy.dispatchSet)(path, value);
    var actions = (0, _reduxEasy.getStore)().getActions();
    expect(actions.length).toBe(1);

    var _actions2 = (0, _slicedToArray2["default"])(actions, 1),
        action = _actions2[0];

    expect(action.type).toEqual(expect.stringMatching(/^@@set /));
    expect(action.payload).toEqual({
      path: path,
      value: value
    });
  });
  test('handleAsyncAction', function (done) {
    var newState = {
      foo: 1,
      bar: true
    };
    var promise = Promise.resolve(newState);
    (0, _reduxEasy.handleAsyncAction)(promise);
    promise.then(function () {
      var actions = (0, _reduxEasy.getStore)().getActions();
      expect(actions.length).toBe(1);

      var _actions3 = (0, _slicedToArray2["default"])(actions, 1),
          action = _actions3[0];

      expect(action.type).toBe('@@async');
      expect(action.payload).toEqual(newState);
      done();
    });
  });
  test('handleAsyncAction that throws', /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(done) {
      var errorMsg, originalTrace, promise;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              errorMsg = 'some message'; // Mock the console.trace function.

              originalTrace = console.trace;

              console.trace = function (msg) {
                expect(msg).toBe(errorMsg);
              };

              promise = Promise.reject(errorMsg);
              _context.prev = 4;
              _context.next = 7;
              return (0, _reduxEasy.handleAsyncAction)(promise);

            case 7:
              done();
              _context.next = 13;
              break;

            case 10:
              _context.prev = 10;
              _context.t0 = _context["catch"](4);
              done.fail('never called console.trace');

            case 13:
              _context.prev = 13;
              // Reset the console.trace function.
              console.trace = originalTrace;
              return _context.finish(13);

            case 16:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[4, 10, 13, 16]]);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }());
});
describe('redux-easy with real store', function () {
  var initialState;
  var version = 0;
  beforeEach(function () {
    mockSessionStorage();
    initialState = (0, _lodash["default"])(INITIAL_STATE); // Clear any previously registered store.

    (0, _reduxEasy.setStore)(null); // Use real store.

    version++;
    (0, _reduxEasy.reduxSetup)({
      initialState: initialState,
      silent: true,
      version: version
    });
  });
  test('dispatchDelete with real store', function () {
    var path = 'bar.baz';
    (0, _reduxEasy.dispatchDelete)(path);
    var actual = (0, _reduxEasy.getPath)(path);
    expect(actual).toBeUndefined();
  });
  test('dispatchFilter with real store', function () {
    var path = 'bar.qux'; // Remove all elements that contain the letter "t".

    var filterFn = function filterFn(element) {
      return !/t/.test(element);
    };

    (0, _reduxEasy.dispatchFilter)(path, filterFn);
    var actual = (0, _reduxEasy.getPath)(path);
    expect(actual).toEqual(['one']);
  });
  test('dispatchFilter passing non-function', function () {
    var filterFn = 'This is not a function.';
    var msg = 'dispatchFilter must be passed a function';
    expect(function () {
      return (0, _reduxEasy.dispatchFilter)('some.path', filterFn);
    }).toThrow(new Error(msg));
  });
  test('dispatchFilter with path to non-array', function () {
    var path = 'bar.baz';

    var filterFn = function filterFn(value) {
      return value;
    };

    var msg = "filterPath can only be used on arrays and ".concat(path, " is not");
    expect(function () {
      return (0, _reduxEasy.dispatchFilter)(path, filterFn);
    }).toThrow(new Error(msg));
  });
  test('dispatchMap with real store', function () {
    var path = 'bar.qux'; // Uppercase all elements.

    var mapFn = function mapFn(element) {
      return element.toUpperCase();
    };

    (0, _reduxEasy.dispatchMap)(path, mapFn);
    var actual = (0, _reduxEasy.getPath)(path);
    expect(actual).toEqual(['ONE', 'TWO', 'THREE']);
  });
  test('dispatchMap passing non-function', function () {
    var filterFn = 'This is not a function.';
    var msg = 'dispatchMap must be passed a function';
    expect(function () {
      return (0, _reduxEasy.dispatchMap)('some.path', filterFn);
    }).toThrow(new Error(msg));
  });
  test('dispatchMap with path to non-array', function () {
    var path = 'bar.baz';

    var filterFn = function filterFn(value) {
      return value;
    };

    var msg = "mapPath can only be used on arrays and ".concat(path, " is not");
    expect(function () {
      return (0, _reduxEasy.dispatchMap)(path, filterFn);
    }).toThrow(new Error(msg));
  });
  test('dispatchPush with real store', function () {
    var path = 'bar.qux'; // Remove all elements that contain the letter "t".

    (0, _reduxEasy.dispatchPush)(path, 'four', 'five');
    var actual = (0, _reduxEasy.getPath)(path);
    expect(actual).toEqual(['one', 'two', 'three', 'four', 'five']);
  });
  test('dispatchPush with path to non-array', function () {
    var path = 'bar.baz';

    var filterFn = function filterFn(value) {
      return value;
    };

    var msg = "pushPath can only be used on arrays and ".concat(path, " is not");
    expect(function () {
      return (0, _reduxEasy.dispatchPush)(path, filterFn);
    }).toThrow(new Error(msg));
  });
  test('dispatchSet with real store', function () {
    var path = 'some.deep.path';
    var value = 'some value';
    (0, _reduxEasy.dispatchSet)(path, value);
    var actual = (0, _reduxEasy.getPath)(path);
    expect(actual).toEqual(value);
  });
  test('dispatchTransform with real store', function () {
    var path = 'bar.baz';
    var initialValue = (0, _reduxEasy.getPath)(path);
    (0, _reduxEasy.dispatchTransform)(path, function (v) {
      return v + 1;
    });
    var newValue = (0, _reduxEasy.getPath)(path);
    expect(newValue).toEqual(initialValue + 1);
  });
  test('dispatchTransform passing non-function', function () {
    var filterFn = 'This is not a function.';
    var msg = 'dispatchTransform must be passed a function';
    expect(function () {
      return (0, _reduxEasy.dispatchTransform)('some.path', filterFn);
    }).toThrow(new Error(msg));
  }); // We are trusting that Redux works and are
  // just including this test for code coverage.

  test('getState', function () {
    // Create a mock Redux devtools store enhancer
    // to get code coverage.
    window.__REDUX_DEVTOOLS_EXTENSION__ = function (next) {
      return next;
    };

    (0, _reduxEasy.addReducer)('setEmail', function (state, value) {
      return _objectSpread({}, state, {
        email: value
      });
    });
    var type = 'setEmail';
    var payload = 'foo@bar.baz';
    (0, _reduxEasy.dispatch)(type, payload);
    var state = (0, _reduxEasy.getStore)().getState();
    expect(state.email).toBe(payload);
  });
  test('getState', function () {
    var state = (0, _reduxEasy.getStore)().getState();
    expect((0, _reduxEasy.getState)()).toEqual(state);
  });
  test('invalid action type', function () {
    var action = {
      type: 'invalid'
    };
    expect(function () {
      return (0, _reduxEasy.reducer)(undefined, action);
    }).toThrowError("no reducer found for action type \"".concat(action.type, "\""));
  });
  test('loadState handles bad JSON', function () {
    global.sessionStorage.setItem(_reduxEasy.STATE_KEY, 'bad json');
    var state = (0, _reduxEasy.loadState)();
    expect(state).toEqual(initialState);
  });
  test('loadState handles state not in sessionStorage', function () {
    global.sessionStorage.removeItem(_reduxEasy.STATE_KEY);
    var state = (0, _reduxEasy.loadState)();
    expect(state).toEqual(initialState);
  });
  test('missing action type', function () {
    var action = {
      wrong: 'some-action'
    };
    expect(function () {
      return (0, _reduxEasy.reducer)(undefined, action);
    }).toThrowError('action object passed to reducer must have type property');
  });
  test('reducer with no state', function () {
    (0, _reduxEasy.addReducer)('noOp', function (state) {
      return state;
    });
    var action = {
      type: 'noOp'
    };
    var state = (0, _reduxEasy.reducer)(undefined, action);
    expect(state).toEqual(initialState);
  });
  test('reduxSetup with no initialState', function () {
    (0, _reduxEasy.setStore)(null); // clears store created in beforeEach

    (0, _reduxEasy.reduxSetup)({}); // not specifying initial state

    var state = (0, _reduxEasy.getState)();
    expect(state).toEqual({});
  });
  test('saveState', function () {
    var state = {
      foo: 2,
      bar: {
        baz: 3
      }
    };
    (0, _reduxEasy.saveState)(state);
    var actual = (0, _reduxEasy.loadState)();
    expect(actual).toEqual(state);
  }); // This verifies that the watch function works with a watchMap
  // which means it passes the correct state path values
  // as a props to a React component.

  test('watch with a watchMap', function (done) {
    var TestComponent = function TestComponent(props) {
      expect(props.baz).toBe(initialState.bar.baz);
      return null; // don't need to actually return JSX in test
    };

    var watchMap = {
      baz: 'bar.baz'
    };
    var HOComponent = (0, _reduxEasy.watch)(TestComponent, watchMap);
    var target = document.createElement('div');
    (0, _reduxEasy.reduxSetup)({
      component: _react["default"].createElement(HOComponent, null),
      initialState: initialState,
      silent: true,
      target: target
    });
    setTimeout(done, 100);
  }); // This verifies that the watch function works with a path.

  test('watch with a path', function (done) {
    var path = 'bar.baz';

    var TestComponent = function TestComponent(props) {
      expect(props.path).toBe(path);
      expect(props.value).toBe(initialState.bar.baz);
      return null; // don't need to actually return JSX in test
    };

    var HOComponent = (0, _reduxEasy.watch)(TestComponent);
    var target = document.createElement('div');
    (0, _reduxEasy.reduxSetup)({
      component: _react["default"].createElement(HOComponent, {
        path: path
      }),
      initialState: initialState,
      silent: true,
      target: target
    }); // Wait for the component to be rendered.

    setTimeout(done, 100);
  }); // This verifies that the watch function works with a list.

  test('watch with a list', function (done) {
    var list = [{
      path: 'foo'
    }, {
      path: 'bar.baz'
    }, {
      path: 'bar.qux'
    }];

    var TestComponent = function TestComponent(props) {
      expect(props.list).toEqual(list);
      expect(props.values.length).toBe(3);

      var _props$values = (0, _slicedToArray2["default"])(props.values, 3),
          v1 = _props$values[0],
          v2 = _props$values[1],
          v3 = _props$values[2];

      expect(v1).toBe(initialState.foo);
      expect(v2).toBe(initialState.bar.baz);
      expect(v3).toEqual(initialState.bar.qux);
      return null; // don't need to actually return JSX in test
    };

    var HOComponent = (0, _reduxEasy.watch)(TestComponent);
    var target = document.createElement('div');
    (0, _reduxEasy.reduxSetup)({
      component: _react["default"].createElement(HOComponent, {
        list: list
      }),
      initialState: initialState,
      silent: true,
      target: target
    }); // Wait for the component to be rendered.

    setTimeout(done, 100);
  }); // This verifies that the watch function throws when not used correctly.

  test('watch used incorrectly', function (done) {
    // Suppress output from console.error.
    var originalError = console.error;

    console.error = function () {
      return undefined;
    };

    var TestComponent = function TestComponent() {
      return null;
    };

    var HOComponent = (0, _reduxEasy.watch)(TestComponent);
    var target = document.createElement('div');

    try {
      (0, _reduxEasy.reduxSetup)({
        component: _react["default"].createElement(HOComponent, null),
        initialState: initialState,
        silent: true,
        target: target
      }); // Trigger a state change so mapState in the watch function will be called.

      (0, _reduxEasy.dispatchSet)('foo', 2); // Wait for the component to be rendered.

      setTimeout(function () {
        return done.fail('expected error when watch is used incorrectly', 100);
      });
    } catch (e) {
      var expectedErrorMsg = 'watched components must have a watchMap, path, or list prop';
      expect(e.message).toBe(expectedErrorMsg);
      done();
    } finally {
      // Reset the console.trace function.
      console.error = originalError;
    }
  });
});
describe('redux-easy with sensitive data', function () {
  beforeEach(function () {
    mockSessionStorage(); // Clear any previously registered store.

    (0, _reduxEasy.setStore)(null);
  });
  test('replacer and reviver', function () {
    var initialState = {
      password: 'secret'
    };

    var replacerFn = function replacerFn(state) {
      var copy = (0, _lodash["default"])(state);
      delete copy.password;
      return copy;
    };

    var safePassword = 'not-secret';

    var reviverFn = function reviverFn(state) {
      var copy = (0, _lodash["default"])(state);
      copy.password = safePassword;
      return copy;
    };

    (0, _reduxEasy.reduxSetup)({
      initialState: initialState,
      replacerFn: replacerFn,
      reviverFn: reviverFn
    });
    var state = (0, _reduxEasy.getState)();
    expect(state.password).toBe(safePassword);
  });
});